//=======================================================================
// Basic C++: collections polymorphes (figures gomtriques)
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: main_Figure.cpp,v 1.1 2005/05/01 16:58:16 jpr Exp $
//=======================================================================

#include <list>
#include <algorithm>
#include <iterator>
#include <iostream>
using namespace std;

#include "stl_extensions.h"
using namespace JPR_STL_EXTENSIONS;

#include "Figure.h"

// Vrifie que pfig pointe sur un objet de type CF ou driv de CF
template <typename CF>
static bool is_kind_of(Figure *pfig)
{
    return dynamic_cast<CF *>(pfig) != 0;
}

// Fonction pour dtruire une figure  travers son pointeur. Ce pointeur est
// mis  0 aprs destruction : il faut donc le PASSER PAR RFRENCE
//
// NB : on pourrait aussi utiliser l'objet-fonction delete_obj, dfini dans
// stl_extensions.h.
static void delete_Figure(Figure *& pfig)
{
    delete pfig;
    pfig = 0;
}

int main()
{
    list<Figure *> lfigs;	// Une liste de pointeurs sur Figure
    print_obj<Figure> doprint(cout, "\n");	// un objet-fonction pour
                                                // imprimer les figures

    // Construcion d'une liste de figures

    lfigs.push_back(new Circle(Point(0.0, 0.0), 2.0));
    lfigs.push_back(new Ellipse(Point(0.0, 1.0), 2.0, 3.0));
    lfigs.push_back(new Square(Point(0.0, 0.0), 2.0));
    lfigs.push_back(new Rectangle(Point(0.0, 1.0), 2.0, 3.0));

    // Affichage de la liste

    cout << "Liste des figures" << endl;
    cout << "-----------------" << endl;
    for_each(lfigs.begin(), lfigs.end(), doprint);
    cout << endl;

    // Tourner toutes les figures de PI/2

    for_each(lfigs.begin(), lfigs.end(), bind2nd(mem_fun(&Figure::rotate), PI/2));
    cout << "Liste des figures aprs rotation" << endl;
    cout << "--------------------------------" << endl;
    for_each(lfigs.begin(), lfigs.end(), doprint);
    cout << endl;

    // Faire une homothtie d'un facteur 2

    for_each(lfigs.begin(), lfigs.end(), bind2nd(mem_fun(&Figure::scale), 2.0));
    cout << "Liste des figures aprs homthtie" << endl;
    cout << "---------------------------------" << endl;
    for_each(lfigs.begin(), lfigs.end(), doprint);
    cout << endl;

    // Copie de la liste de figures
    list<Figure *> lfigs_save = lfigs;

    // Enlever toute les ellipses (et drives) de la liste : solution INCORRECTE
 
    remove_if(lfigs.begin(), lfigs.end(), is_kind_of<Ellipse>);
    cout << "Liste des figures aprs enlvement INCORRECT des ellipses" << endl;
    cout << "---------------------------------------------------------" << endl;
    for_each(lfigs.begin(), lfigs.end(), doprint);
    cout << endl;

    // Enlever toute les ellipses (et drives) de la liste : solution CORRECTE
 
    // Remet la liste dans l'tat prcdent
    lfigs = lfigs_save;

    // D'abord dtruire toutes les ellipses et drives
    // Le pointeur est mis  0 par delete_Figure()
    for_each_if(lfigs.begin(), lfigs.end(), delete_Figure, is_kind_of<Ellipse>); 

    // Ensuite enlever tous les pointeurs nuls de la liste : l'algorithme
    // remove() rarrange la liste pour que les pointeurs non nuls soient en
    // tte, et la fonction-membre erase() enelve les cellules finales.
    // Attention : le cast de 0 est impratif ! (Pourquoi  votre avis ?)

    lfigs.erase(remove(lfigs.begin(), lfigs.end(), static_cast<Figure *>(0)),
		lfigs.end());
    cout << "Liste des figures aprs enlvement correct des ellipses" << endl;
    cout << "-------------------------------------------------------" << endl;
    for_each(lfigs.begin(), lfigs.end(), doprint);
    cout << endl;

    return 0;
}
