//=======================================================================
// Basic C++: classe Date
//      Specification of class Date
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: Date.h,v 1.2 2004/11/01 14:26:12 jpr Exp $
//=======================================================================

#ifndef _DATE_H_
#define _DATE_H_

#include <iostream>
using namespace std;

class Date
{
private:
    int _year;  // since 1900
    int _month; // 1 to 12
    int _day;   // 1 to 31

public:
    // Static member functions
    static Date today();
    static bool is_leap(int y);

    // Exception
    class Bad_Year {};
    class Bad_Month {};
    class Bad_Day {};
    class Bad_Input {};

    // Constructor
    Date();     // Default constructor: yield today
    Date(int y, int m, int d);

    // Accessors
    int year() const {return _year;}
    int month() const {return _month;}
    int day() const {return _day;}

    // Arithmetic operations
    Date operator+(int n) const;        // add n days
    Date operator-(int n) const;        // remove n days
    friend int operator-(Date d1, Date d2); // number of days between 2 dates

    Date& operator++();         // prefix
    Date operator++(int);       // postfix
    Date& operator--();         // prefix
    Date operator--(int);       // psotfix

    Date& operator+=(int n);
    Date& operator-=(int n);

    // Relational operators
    friend bool operator==(Date d1, Date d2);
    friend bool operator!=(Date d1, Date d2);
    friend bool operator<(Date d1, Date d2);
    friend bool operator<=(Date d1, Date d2);
    friend bool operator>(Date d1, Date d2);
    friend bool operator>=(Date d1, Date d2);

    // IO operations
    friend ostream& operator<<(ostream& os, Date dat);
    friend istream& operator>>(istream& os, Date& dat);
};


#endif

