//=======================================================================
// Basic C++: a simple class template List
//      Specification
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: List.h,v 1.3 2005/02/20 15:26:03 jpr Exp $
//=======================================================================

#ifndef _LIST_H_
#define _LIST_H_

#include <iostream>
#include <cassert>
using namespace std;

#include "common_defs.h"

template <typename ELEM>
class List;
template <typename ELEM>
ostream& operator<<(ostream&, const List<ELEM>&);

template <typename ELEM>
class List
{
private:
    
   // The list Cell type 
    
    struct Cell
    {
        ELEM val;
        Cell *next;
        Cell(ELEM v, Cell *n = 0) : val(v), next(n) {}
        ~Cell() {delete next;} // Recursive destructor
    };

    // 
    Cell *head;         // Pointer to first cell
    Cell *tail;         // Pointer to last cell

public:
    // Exception

    class Empty {};

    // Construction and destruction

    List() : head(0), tail(0) {}

    // Destroy first cell (recursive destruction)
    // It's not necessary to be virtual, but if it is not we get a
    // warning in Queu ! 
    virtual ~List() {delete head;} 

    // Copy constructor and copy assignment

    List(const List&);
    List& operator=(const List&);

    // Accessor ?
    
    bool is_empty() const {return head == 0;}

    // Adding elements to the list
    
    void append(ELEM);  // at the end
    void prepend(ELEM);  // at the beginning
    void insert(ELEM);  // at its place, according to ELEM::operator<

    // Removing elements from the list en tete de liste
    
    ELEM get_first();

    // Display (obey the g++ warning message and add <>!)
    
    friend ostream& operator<< <>(ostream&, const List&);
};

// Inclusion model for templates
#include "List.cpp"

#endif
