//=======================================================================
// Basic C++: a simple class template Queue
//      Specification and implementation
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: Queue.h,v 1.3 2005/02/20 14:50:29 jpr Exp $
//=======================================================================

#ifndef _QUEUE_H_
#define _QUEUE_H_

#include "List.h"

//----------------------------------------------------------------------- 
// A FIFO queue using List as a base implementation
//-----------------------------------------------------------------------
// Protected inheritance is used to inherit the code but not the (public)
// nterface. Protected and not private since Priority_Queue derives from 
// Queue and must have accest to List::insert().
//
// Functions put() and get() are simply defined by reusing append() and 
// get_first(). 
//-----------------------------------------------------------------------

template <typename ELEM>
class Queue : protected List<ELEM>
{
public:

    // Always put at the end an retrieve from the beginning. Function put()
    // needs to be virtual because of its redefinition in Priority_Queue
    virtual void put(ELEM e) {append(e);}
    ELEM get() {return List<ELEM>::get_first();}

    // Reexport publicly this members (which became private owing to private
    // inheritance)
    using List<ELEM>::is_empty;

    // Because of private derivation we need to override operator<<
    // Note the cast to a reference, to avoid copying
    friend ostream& operator<<(ostream& os, const Queue& q)
    {
        return os << static_cast<const List<ELEM>&>(q);
    }
};

#endif
