//=======================================================================
// Basic C++: a simple class template Stack
//      Specification and implementation
//-----------------------------------------------------------------------
// Jean-Paul Rigault --- Copyright 2003
// $Id: Stack.h,v 1.3 2005/02/20 14:50:29 jpr Exp $
//=======================================================================

#ifndef _STACK_H_
#define _STACK_H_

#include "List.h"

//----------------------------------------------------------------------- 
// A LIFO stack using List as a base implementation
//-----------------------------------------------------------------------
// Private inheritance is used to inherit the code but not the (public)
// nterface.USing private inheritance means that we do not expect to 
// derive a class from Stack or that if we derive one, the latter won't 
// need access to List member-functions.).
//
// Functions push() and pop() are simply defined by reusing prepend() and 
// get_first(). 
//-----------------------------------------------------------------------

template <typename ELEM>
class Stack : private List<ELEM>
{
public:

    // Always add at and retrieve from the beginning  
    void push(ELEM e) {prepend(e);}
    ELEM pop() {return List<ELEM>::get_first();}

    // Reexport publicly this members (which became private owing to private
    // inheritance)
    using List<ELEM>::is_empty;

    // Because of private derivation we need to override operator<<
    // Note the cast to a reference, to avoid copying
    friend ostream& operator<<(ostream& os, const Stack& s)
    {
        return os << static_cast<const List<ELEM>&>(s);
    }
};

#endif

